/**
 * @file StereoAmp_click_ARM.c
 * @brief <h2> StereoAmp click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * StereoAmp click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        3 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    STM32F207VG
 * -<b> Dev. Board    </b> :    EasyMx PRO v7 for STM32
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    StereoAmp click board
 * -<b> SW            </b> :    mikroC PRO for ARM v4.9.0
 *
 */

/******************************************************************************
* Preprocessors Constants
*******************************************************************************/
#define AUTO_WR_BIT

/******************************************************************************
* Includes
*******************************************************************************/
#include "StereoAmp.h"

/******************************************************************************
* Typedefs
*******************************************************************************/
typedef struct volume_widget {
    uint8_t active;
    char label;
    uint16_t x;
    uint16_t y;
    uint8_t value;
} volume_widget;

/******************************************************************************
* Variable Definitions
*******************************************************************************/
// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

uint8_t volume_control_L = 15;
uint8_t volume_control_R = 15;

uint8_t buttonstate_right = 0xFF;
uint8_t buttonstate_left = 0xFF;
uint8_t buttonstate_up = 0xFF;
uint8_t buttonstate_down = 0xFF;

uint8_t active_left_control = 0xFF;


volume_widget volume_widget_L;
volume_widget volume_widget_R;

/******************************************************************************
* Public Functions
*******************************************************************************/
void Write_Text(const char* text, uint8_t x, uint8_t y) {
    TFT_Set_Font(TFT_defaultFont, CL_GRAY, FO_HORIZONTAL);
    TFT_Write_Text(text, x, y );
}

void Erase_Text(const char* text, uint8_t x, uint8_t y) {
    TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text(text, x, y );
}

void Volume_Widget_Init(volume_widget* widget) {
    uint8_t i;
    uint16_t x = widget->x;
    uint16_t y = widget->y;

    TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
    TFT_Set_Pen(CL_BLACK, 0);

    TFT_Rectangle(x, y, x + 198, y + 15);
    TFT_Set_Font(TFT_defaultFont, CL_GRAY, FO_HORIZONTAL);

    TFT_Write_Text(gain_text[widget->value], x + 28, y );
    TFT_Write_Char(widget->label, x + 18, y );

    if(widget->active) {
        TFT_Set_Brush(1, CL_GRAY, 0, 0, 0, 0);
        TFT_Rectangle(x + 5, y + 5, x + 10, y + 10);
    }

    TFT_Set_Pen(CL_GRAY, 1);
    TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
    TFT_Rectangle(x + 97, y, x + 195, y + 15);

    TFT_Set_Pen(CL_GRAY, 2);
    TFT_V_Line(y + 2, y + 13, x + 100);
    for(i = 1; i <= widget->value; i++) {
        TFT_V_Line(y + 2, y + 13, x + 100 + i * 3);
    }

}


void Write_Amp_Reg(uint8_t amp_addr, uint8_t reg_addr, uint8_t reg_data) {
    char data_write = (reg_addr & REG_ADDR_MASK) | (reg_data & DATA_MASK);
    delay_ms(1);
    I2C1_Start();
    I2C1_Write(amp_addr, &data_write, 1, END_MODE_STOP);
}


void MCU_Init() {
    // IMPORTANT: Toggle I2C lines (PB6, PB7), before initialising I2C module to enable amplifiers.
    GPIO_Digital_Output(&GPIOB_ODR, _GPIO_PINMASK_6 |_GPIO_PINMASK_7);
    GPIOB_ODR = 0xC0;
    Delay_ms(300);
    GPIOB_ODR = 0x00;
    Delay_ms(300);

    GPIO_Digital_Input(&GPIOA_IDR, _GPIO_PINMASK_6); 
    GPIO_Digital_Input(&GPIOB_IDR, _GPIO_PINMASK_5);
    GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_2 | _GPIO_PINMASK_4);

    //I2C1_Init();
    I2C1_Init_Advanced(I2C_CLK, &_GPIO_MODULE_I2C1_PB67);

    Delay_ms(800);

    TFT_Init_ILI9341_8bit(320, 240);
    TFT_Fill_Screen(CL_BLACK);
}

void StereoAmp_Init() {
    // Power On and In1.
    Write_Amp_Reg(LEFT_SPEAKER, MODE_CONTROL_REG, POWER_ON | INPUT1);
    Write_Amp_Reg(RIGHT_SPEAKER, MODE_CONTROL_REG, POWER_ON | INPUT1);

    // Volume min.
    Write_Amp_Reg(LEFT_SPEAKER, VOLUME_CONTROL1_REG, volume_control_L);
    Write_Amp_Reg(RIGHT_SPEAKER, VOLUME_CONTROL1_REG, volume_control_R);

    // FAULT.
    Write_Amp_Reg(LEFT_SPEAKER, FAULT_DETECTION_REG, OUTPUT_OPEN | OUTPUT_SHT | RAIL_SHT | OVF | TSD);
    Write_Amp_Reg(RIGHT_SPEAKER, FAULT_DETECTION_REG, OUTPUT_OPEN | OUTPUT_SHT | RAIL_SHT | OVF | TSD);

    // Diagnostic Enable.
    Write_Amp_Reg(LEFT_SPEAKER, DIAGNOSTIC_CONTROL_REG, DG_EN | DG_RESET | ILIMIT);
    Write_Amp_Reg(RIGHT_SPEAKER, DIAGNOSTIC_CONTROL_REG, DG_EN | DG_RESET | ILIMIT);
}


void Set_Volume(char amp_addr, char value) {
    Write_Amp_Reg(amp_addr, VOLUME_CONTROL1_REG, value);
}

uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state) {
    uint8_t new_state = Button(port, pin, 1, 1);
    if(*state == 0xFF && new_state == 0x00 ) {
        *state = new_state;
        return 0xFF;
    }

    *state = new_state;
    return 0x00;
}

uint8_t Button_Hold(unsigned int *port, unsigned int pin, uint8_t* state) {
    uint8_t new_state = Button(port, pin, 1, 1);
    if(*state == 0x00 && new_state == 0x00 ) {
        *state = new_state;
        return 0xFF;
    }

    *state = new_state;
    return 0x00; 
}

void Volume_Widget_Update(volume_widget* widget, uint8_t value, uint8_t active) {
    uint8_t i;
    uint16_t x = widget->x;
    uint16_t y = widget->y;

    if(value != widget->value) {
        Erase_Text(gain_text[widget->value], x + 28, y );

        if(value < widget->value) {
            TFT_Set_Pen(CL_BLACK, 2);
            TFT_V_Line(y + 2, y + 13, x + 100 + widget->value * 3);
        }

        if(value > widget->value) {
            TFT_Set_Pen(CL_GRAY, 2);
            TFT_V_Line(y + 2, y + 13, x + 100 + value * 3);
        }

        widget->value = value;

        Write_Text(gain_text[widget->value], x + 28, y );
    }

    if(widget->active != active) {
        TFT_Set_Pen(CL_BLACK, 0);
        if(active) 
            TFT_Set_Brush(1, CL_GRAY, 0, 0, 0, 0);
        else 
            TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
        
        TFT_Rectangle(x + 5, y + 5, x + 10, y + 10);
        widget->active = active;
    }
}

uint8_t Clamp(int16_t value, int16_t min, int16_t max) {
    if(value < min) return min;
    if(value > max) return max;

    return value;
}


void main() {
    MCU_Init();
    
    Write_Text("Initialising...", 10, 20 );
    StereoAmp_Init();

    Erase_Text("Initialising...", 10, 20 );
    
    Write_Text("StereoAmp Click - Example", 10, 20 );
    Write_Text("Note: Use Joystic for controlling volume.", 10, 120 );
    
    volume_widget_L.active = 0xFF;
    volume_widget_L.label = 'R';
    volume_widget_L.x = 10;
    volume_widget_L.y = 50;
    volume_widget_L.value = volume_control_L;

    volume_widget_R.active = 0x00;
    volume_widget_R.label = 'L';
    volume_widget_R.x = 10;
    volume_widget_R.y = 75;
    volume_widget_R.value = volume_control_R;

    Volume_Widget_Init(&volume_widget_L);
    Volume_Widget_Init(&volume_widget_R);

    while(1) {
        if(Button_Hold(&GPIOA_IDR, 6, &buttonstate_right)) {
            if(active_left_control) {
                volume_control_L = Clamp(volume_control_L + 1, 0, 31);
                Set_Volume(LEFT_SPEAKER, volume_control_L);
            } else {
                volume_control_R = Clamp(volume_control_R + 1, 0, 31);
                Set_Volume(RIGHT_SPEAKER, volume_control_R);
            }
        }

        if(Button_Hold(&GPIOD_IDR, 2, &buttonstate_left)) {
            if(active_left_control) {
                volume_control_L = Clamp(volume_control_L - 1, 0, 31);
                Set_Volume(LEFT_SPEAKER, volume_control_L);
            } else {
                volume_control_R = Clamp(volume_control_R - 1, 0, 31);
                Set_Volume(RIGHT_SPEAKER, volume_control_R);
            }
        }

        if(Button_Pressed(&GPIOD_IDR, 4, &buttonstate_up)) 
            active_left_control = 0xFF;
    
        if(Button_Pressed(&GPIOB_IDR, 5, &buttonstate_down)) 
            active_left_control = 0x00;
        
        Volume_Widget_Update(&volume_widget_L, volume_control_L, active_left_control);
        Volume_Widget_Update(&volume_widget_R, volume_control_R, ~active_left_control);
        
        Delay_ms(POLLING_PERIOD);
    }
}
/*************** END OF FUNCTIONS *********************************************/